@@ -32,7 +32,7 @@ eval 'use Net::OpenSSH; use IO::Pty; $haveit{"openssh"} = 1; 1';
 eval 'use Net::FSP; $haveit{"fsp"} = 1; 1';
 eval 'use Net::FTPSSL; $haveit{"ftpssl"} = 1; 1';
 
-our $VERSION = '1.00';
+our $VERSION = '1.01';
 
 sub new
 {
@@ -290,10 +290,15 @@ sub haveSFTP
 
 sub haveModule
 {
+	my $self = shift;
 	my $modul = shift;
 
 	my $modHash = &haveModules();
-	return (defined $modHash->{$modul}) ? $modHash->{$modul} : 0;
+	if ($modul =~ /^Net\:\:/) {
+		return (defined $modHash->{$modul}) ? $modHash->{$modul} : 0;
+	}
+	$modul =~ tr/A-Z/a-z/;
+	return (defined $haveit{$modul}) ? $haveit{$modul} : 0;
 }
 
 sub haveModules
