@@ -209,8 +209,6 @@ sub get    #(Remote, => Local)
 
 	return undef  unless (@_ >= 1);
 	my @args = @_;
-print STDERR "-get: args=".join('|',@args)."= ref=".ref($args[1])."= /ref=".ref(\$args[1])."= content=".$$args[1]."=\n";
-print STDERR "-arg1 dereferenced =$args[1]=\n";
 	if (@args >= 2)
 	{
 		$args[1] = \$_[1]  if (ref(\$args[1]) =~ /GLOB/io);
@@ -428,4 +426,24 @@ sub chmod
 	return undef;
 }
 
+sub method
+{
+	my $self = shift;
+	my $method = shift;
+
+	for (my $i=0;$i<scalar(@_);$i++)
+	{
+		$_[$i] = "'" . $_[$i] . "'"  unless ($_[$i] =~ /^['"]/o || $_[$i] =~ /^[\d\.\+\-]+$/o);
+	}
+	my $res;
+	my $xeq = " \$res = \$self->{xftp}->$method(".join(',', @_).")";
+	eval $xeq;
+	if ($@)
+	{
+		$self->{xftp_lastmsg} = "method Net::FSP::$method(".join(',',@_).") returned $@";
+		return undef;
+	}
+	return $res;
+}
+
 1
