@@ -424,6 +424,7 @@ sub message
 {
 	my $self = shift;
 
+	$self->{xftp_lastmsg} ||= '';
 	chomp $self->{xftp_lastmsg};
 	my $res = $self->{xftp}->status;
 	return (length $res) ? ("$res - ".$self->{xftp_lastmsg}) : $self->{xftp_lastmsg};
@@ -558,4 +559,24 @@ sub getPermStr
 	return $pstr;
 }	
 
+sub method
+{
+	my $self = shift;
+	my $method = shift;
+
+	for (my $i=0;$i<scalar(@_);$i++)
+	{
+		$_[$i] = "'" . $_[$i] . "'"  unless ($_[$i] =~ /^['"]/o || $_[$i] =~ /^[\d\.\+\-]+$/o);
+	}
+	my $res;
+	my $xeq = " \$res = \$self->{xftp}->$method(".join(',', @_).")";
+	eval $xeq;
+	if ($@)
+	{
+		$self->{xftp_lastmsg} = "method Net::SFTP::Foreign::$method(".join(',',@_).") returned $@";
+		return undef;
+	}
+	return $res;
+}
+
 1
