@@ -129,6 +129,7 @@ sub ls
 			next  if (!$showall && $t =~ /^\.[^\.]/o);
 			push (@dirlist, $t);
 		}
+		closedir D;
 	}
 	else
 	{
@@ -217,7 +218,7 @@ sub dir
 	}
 	else
 	{
-		my @d = $showall ? `ls -la $realpath` : `ls -l $realpath`;
+		my @d = $showall ? `ls -la "$realpath"` : `ls -l "$realpath"`;
 		if (@d)
 		{
 			shift @d  if ($d[0] =~ /^total \d/o);   #REMOVE "TOTAL" LINE.
