@@ -33,6 +33,7 @@ sub new_openssh
 	}
 	my $saveEnvHome = $ENV{HOME};
 	$ENV{HOME} = $xftp_args{home}  if ($xftp_args{home});
+
 	$xftp->{xftp} = Net::OpenSSH->new($host, %args);
 	$xftp->{xftp_lastmsg} = $@  if ($@);
 	$ENV{HOME} = $saveEnvHome || '';
@@ -52,7 +53,9 @@ sub new_openssh
 			$@ = $xftp->{xftp_lastmsg};
 			return undef;
 		}
+		chomp $cwd;
 		$xftp->{cwd} = $cwd;
+
 		return $xftp;
 	}
 	return undef;
@@ -69,18 +72,13 @@ sub new_openssh
 		#NEXT 22 ADDED 20060815 TO FIX RELATIVE PATH CHANGES.
 		if ($cwd !~ m#^\/# && $self->{cwd} && $self->{cwd} !~ /^\./o)
 		{
-			if ($self->{cwd} =~ m#\/$#o)
-			{
-				$cwd = $self->{cwd} . $cwd;
-			}
-			else
-			{
-				$cwd = $self->{cwd} . '/' . $cwd;
-			}
+			$cwd = ($self->{cwd} =~ m#\/$#o) ? $self->{cwd} . $cwd : $self->{cwd} . '/' . $cwd;
 		}
 		elsif ($cwd eq '..')
 		{
-			$cwd = $self->{xftp}->capture("cd \"$self->{cwd}\"");
+#DOES NOT CHANGE PWD TO CWD, SO WE KEEP	OURSELVES?!			$self->{xftp}->capture("cd \"$$self{cwd}\"");
+#DOES NOT CHANGE PWD TO CWD, SO WE KEEP	OURSELVES?!			$cwd = $self->{xftp}->capture('pwd');
+			$cwd = $self->{cwd};  #DOES NOT CHANGE PWD TO CWD, SO WE KEEP	OURSELVES?!
 			chomp $cwd;
 			chop $cwd  if ($cwd =~ m#\/$#o);
 			$cwd =~ s#\/[^\/]+$##o;
@@ -88,17 +86,20 @@ sub new_openssh
 		}
 		elsif ($cwd eq '.')
 		{
-			$cwd = $self->{xftp}->capture('pwd');
+#DOES NOT CHANGE PWD TO CWD, SO WE KEEP	OURSELVES?!			$cwd = $self->{xftp}->capture('pwd');
+			$cwd = $self->{cwd};  #DOES NOT CHANGE PWD TO CWD, SO WE KEEP	OURSELVES?!
 		}
-		$fullwd = $self->{xftp}->capture("cd \"$self->{cwd}\"");
+		$self->{xftp}->capture("cd \"$cwd\"");
+		$fullwd = $self->{xftp}->capture('pwd');
 		if (!$fullwd || $self->{xftp}->error)
 		{
 			$self->{xftp_lastmsg} = $self->{xftp}->error || "xFTP:cwd() failed for unknown reason!"
 					. ')!';
 			return undef;
 		}
-		chomp $fullwd;
-		$self->{cwd} = $fullwd;
+#DOES NOT CHANGE PWD TO CWD, SO WE KEEP	OURSELVES?!	chomp $fullwd;
+#DOES NOT CHANGE PWD TO CWD, SO WE KEEP	OURSELVES?!		$self->{cwd} = $fullwd;
+		$self->{cwd} = $cwd;   #DOES NOT CHANGE PWD TO CWD, SO WE KEEP	OURSELVES?!
 		return 1;
 	}
 
@@ -166,17 +167,20 @@ sub ls
 	my $self = shift;
 	my $path = shift || '';
 	my $showall = shift || 0;
+
 	my @dirlist;
 	my $realpath = $path || $self->{cwd} || '.';
+
 	$realpath = $self->{cwd} . '/' . $realpath  unless ($realpath =~ m#^(?:[a-zA-Z]\:|\/)#o);
 	my @dirHash = $self->{xftp}->capture("ls \"$realpath\"");
-	if (!defined(@dirHash) || $self->{xftp}->error)
+	if (!@dirHash || $self->{xftp}->error)
 	{
 		$self->{xftp_lastmsg} = $self->{xftp}->error || "xFTP:ls() failed for unknown reason!";
 		my $err = $self->{xftp}->error;
 		return undef;
 	}
 	return  unless (defined $dirHash[0]);     #ADDED 20071024 FOR CONSISTENCY.
+
 #	shift (@dirHash)  if ($dirHash[0]->{longname} =~ /^total \d/o);  #REMOVE TOTAL LINE!
 	my $t;
 	@dirlist = ();
@@ -202,17 +206,17 @@ sub dir
 	my $self = shift;
 	my $path = shift || '';
 	my $showall = shift || 0;
+
 	my @dirlist;
 	my $realpath = $path || $self->{cwd} || '.';
 	$realpath = $self->{cwd} . '/' . $realpath  unless ($realpath =~ m#^(?:[a-zA-Z]\:|\/)#o);
 	my @dirHash = $self->{xftp}->capture("ls -l \"$realpath\"");
-	if (!defined(@dirHash) || $self->{xftp}->error)
+	if (!@dirHash || $self->{xftp}->error)
 	{
 		$self->{xftp_lastmsg} = $self->{xftp}->error || "xFTP:dir() failed for unknown reason!";
 		my $err = $self->{xftp}->error;
 		return undef;
 	}
-#	shift (@dirHash)  if ($dirHash[0]->{longname} =~ /^total \d/o);  #REMOVE TOTAL LINE!
 	my $t;
 	@dirlist = ();
 	foreach my $t (sort @dirHash)
@@ -235,7 +239,7 @@ sub pwd  #GET AND RETURN THE "CURRENT" DIRECTORY.
 {
 	my $self = shift;
 
-	return $self->{cwd};
+	return $self->{cwd} || $self->{xftp}->capture('pwd');
 }
 
 sub get    #(Remote, => Local)
@@ -285,11 +289,18 @@ sub get    #(Remote, => Local)
 	}
 	else
 	{
-		$ok = $self->{xftp}->scp_get(@args);
-		if (!$ok || $self->{xftp}->error)
+		my $ok = $self->{xftp}->scp_get(@args);
+		my $bang = $!;
+		my $err = $self->{xftp}->error;
+		if (!$ok || $err || $bang)
 		{
-			$self->{xftp_lastmsg} = $self->{xftp}->error || "xFTP:get() failed - unknown reason!";
-			return undef;
+			$self->{xftp_lastmsg} = $err || $bang || "xFTP:get() failed - unknown reason!";
+			if ($SIG{CHLD} eq 'IGNORE' && $err eq 'scp failed: child exited with code 1')
+			{
+				$self->{xftp_lastmsg} = $bang;
+				return 1;
+			}
+			return $ok ? 1 : undef;
 		}
 	}
 	return $ok ? 1 : undef;
@@ -343,11 +354,18 @@ sub put    #(LOCAL => REMOTE) SFTP returns OK=1 on SUCCESS.
 	}
 	else
 	{
-		$ok = $self->{xftp}->scp_put(@args);
-		if (!$ok || $self->{xftp}->error)
+		my $ok = $self->{xftp}->scp_put(@args);
+		my $bang = $!;
+		my $err = $self->{xftp}->error;
+		if (!$ok || $err || $bang)
 		{
-			$self->{xftp_lastmsg} = $self->{xftp}->error || "xFTP:put() failed - unknown reason!";
-			return undef;
+			$self->{xftp_lastmsg} = $err || $bang || "xFTP:put() failed - unknown reason!";
+			if ($SIG{CHLD} eq 'IGNORE' && $err eq 'scp failed: child exited with code 1')
+			{
+				$self->{xftp_lastmsg} = $bang;
+				return 1;
+			}
+			return $ok ? 1 : undef;
 		}
 	}
 	return $ok ? 1 : undef;
@@ -371,6 +389,7 @@ sub delete       #RETURNED OK=2 WHEN LAST FAILED.
 sub rename
 {
 	my $self = shift;
+
 	return undef  unless (@_ == 2);
 
 	my ($oldfile, $newfile) = @_;
@@ -391,8 +410,8 @@ sub mkdir
 	my $self = shift;
 	my $path = shift;
 	my $tryRecursion = shift||0;
-	$path =~ s#[\/\\]$##o  unless ($path eq '/');
 
+	$path =~ s#[\/\\]$##o  unless ($path eq '/');
 	my @pathStack;
 	my $ok = '';
 	my $orgPath = $path;
@@ -443,6 +462,7 @@ sub rmdir
 {
 	my $self = shift;
 	my $path = shift;
+
 	$path =~ s#[\/\\]$##o  unless ($path eq '/');
 
 	$path = $self->{cwd} . '/' . $path  unless ($path =~ m#^(?:[a-zA-Z]\:|\/)#o);
@@ -459,9 +479,10 @@ sub message
 {
 	my $self = shift;
 
+	return ''  unless ($self->{xftp_lastmsg});
 	chomp $self->{xftp_lastmsg};
-	my $res = $self->{xftp}->error;
-	return ($self->{xftp_lastmsg} =~ /^\s*$res[0]/) ? $self->{xftp_lastmsg} : $res;
+
+	return $self->{xftp_lastmsg};
 }
 
 sub mdtm
@@ -472,7 +493,7 @@ sub mdtm
 	my $ok;
 #	$path = $self->{cwd} . '/' . $path  unless ($path =~ m#^(?:[a-zA-Z]\:|\/)#o);
 	my @dirHash = $self->{xftp}->capture("stat \"$path\"");
-	if (!defined(@dirHash) || $self->{xftp}->error)
+	if (!@dirHash || $self->{xftp}->error)
 	{
 		$self->{xftp_lastmsg} = $self->{xftp}->error || "xFTP:mdtm() failed for unknown reason!";
 		return undef;
@@ -497,7 +518,7 @@ sub size
 
 #	$path = $self->{cwd} . '/' . $path  unless ($path =~ m#^(?:[a-zA-Z]\:|\/)#o);
 	my @dirHash = $self->{xftp}->capture("ls -l \"$path\"");
-	if (!defined(@dirHash) || $self->{xftp}->error)
+	if (!@dirHash || $self->{xftp}->error)
 	{
 		$self->{xftp_lastmsg} = $self->{xftp}->error || "xFTP:size() failed for unknown reason!";
 		return undef;
@@ -512,13 +533,18 @@ sub isadir
 	my $self = shift;
 	my $path = shift;
 
-	my @dirHash = $self->{xftp}->capture("ls -l \"$path\"");
-	if (!defined(@dirHash) || $self->{xftp}->error)
+#x	$path = $self->{cwd} . $path  unless ($path =~ m#^\/\.#o);
+	unless ($path =~ m#^[\/\.]#o)
+	{
+		$path = ($self->{cwd} =~ m#\/$#o) ? $self->{cwd} . $path : $self->{cwd} . '/' . $path;
+	}
+	my @dirHash = $self->{xftp}->capture("ls -ld \"$path\"");
+	if (!@dirHash || $self->{xftp}->error)
 	{
 		$self->{xftp_lastmsg} = $self->{xftp}->error || "xFTP:isadir() failed for unknown reason!";
 		return undef;
 	}
-	return 1  if ($dirHash[0] =~ /^total\s+\d+$/o || $dirHash[$#dirHash] =~ /^total\s+\d+$/o);
+	return 1  if ($dirHash[0] =~ /^(?:dr|total\s+\d+)/o || $dirHash[$#dirHash] =~ /^total\s+\d+$/o);
 	return 0;
 }
 
@@ -539,4 +565,24 @@ sub chmod
 	return 1;
 }
 
+sub method
+{
+	my $self = shift;
+	my $method = shift;
+
+	for (my $i=0;$i<scalar(@_);$i++)
+	{
+		$_[$i] = "'" . $_[$i] . "'"  unless ($_[$i] =~ /^['"]/o || $_[$i] =~ /^[\d\.\+\-]+$/o);
+	}
+	my $res;
+	my $xeq = " \$res = \$self->{xftp}->$method(".join(',', @_).")";
+	eval $xeq;
+	if ($@)
+	{
+		$self->{xftp_lastmsg} = "method Net::OpenSSH::$method(".join(',',@_).") returned $@";
+		return undef;
+	}
+	return $res;
+}
+
 1
