@@ -576,4 +576,24 @@ sub chmod
 	return 1;
 }
 
+sub method
+{
+	my $self = shift;
+	my $method = shift;
+
+	for (my $i=0;$i<scalar(@_);$i++)
+	{
+		$_[$i] = "'" . $_[$i] . "'"  unless ($_[$i] =~ /^['"]/o || $_[$i] =~ /^[\d\.\+\-]+$/o);
+	}
+	my $res;
+	my $xeq = " \$res = \$self->{xftp}->$method(".join(',', @_).")";
+	eval $xeq;
+	if ($@)
+	{
+		$self->{xftp_lastmsg} = "method Net::SFTP::$method(".join(',',@_).") returned $@";
+		return undef;
+	}
+	return $res;
+}
+
 1
